import 'package:iroute/iroute.dart';
import 'package:flutter/material.dart';

// Widget that contains the AdaptiveNavigationScaffold
class IrouteAppShell extends StatefulWidget {
  IrouteAppShell({required this.pageList, required this.routes});
  final PageList pageList;
  final Map<String, RouteBuilder> routes;
  @override
  IrouteAppShellState createState() =>
      IrouteAppShellState(pageList: pageList, routes: routes);
}

class IrouteAppShellState extends State {
  IrouteAppShellState({required this.pageList, required this.routes})
      : _routerDelegate = IRoute(routes: routes);

  IRouteMaster? root;
  final IRoute _routerDelegate;
  final PageList pageList;
  final Map<String, RouteBuilder> routes;
  ChildBackButtonDispatcher? _backButtonDispatcher;
  int _index = 0;

  @override
  void initState() {
    super.initState();
  }

  @override
  void didUpdateWidget(covariant IrouteAppShell oldWidget) {
    super.didUpdateWidget(oldWidget);
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
// Defer back button dispatching to the child router
    _backButtonDispatcher = Router.of(context)
        .backButtonDispatcher!
        .createChildBackButtonDispatcher();
  }

  @override
  Widget build(BuildContext context) {
// Claim priority, If there are parallel sub router, you will need
// to pick which one should take priority;

    _backButtonDispatcher!.takePriority();
    //似乎在initState中较好？但获取IRouteMaster需要context
    //所以仅仅在root为null的时候执行一次
    if (root == null) {
      root = IRouteMaster.of(context);
      _routerDelegate.root = root;
      _routerDelegate.pageList = (widget as IrouteAppShell).pageList;

      //如果没有add，则无论——routerDelegate还是root，通知的时候这里都不会重绘
      //只有在此setState才会触发重绘
      _routerDelegate.addListener(() {
        //if (_index != _routerDelegate.selectedPage ) {
        _index = _routerDelegate.selectedPage;
        setState(() => {});
        //}
      });
    }

    //_index = root!.currentConfiguration == '/settings' ? 1 : 0;
    //print('index=$_index');
    List<BottomNavigationBarItem> items = [];
    pageList.pages.forEach((element) {
      items.add(element.bottom);
    });

    return Scaffold(
      body: Router(
        routerDelegate: _routerDelegate,
        backButtonDispatcher: _backButtonDispatcher,
      ),
      bottomNavigationBar: BottomNavigationBar(
        items: items,
        currentIndex: _index, //appState.selectedIndex,
        onTap: (newIndex) {
          //这里不做重绘制，一旦做了，则会出现异常
          //_index = newIndex;
          _routerDelegate.selectedPage = newIndex;
          // setState(() {});
          //_routerDelegate.push(newIndex == 0 ? '/' : '/settings');

          //root的重绘，仅解决了更新浏览器地址的问题
          //root!.currentConfiguration = newIndex == 0 ? '/' : '/settings';

          //由于没有重绘，这里针对appshell重绘。
          //this.setState(() {});

          //问题出在这里，单独运行它会出错。
          //_routerDelegate!.push(newIndex == 0 ? '/' : '/settings');

          //appState.selectedIndex = newIndex;
        },
      ),
    );
  }
}
